@@ -69,7 +69,7 @@ static void im_uim_class_init (GtkIMContextClass *class);
 
 static void show_preedit(GtkIMContext *ic, GtkWidget *preedit_label);
 
-static void helper_disconnect_cb(void);
+static void im_uim_helper_disconnect_cb(void);
 static void helper_read_cb(gpointer p, int fd, GdkInputCondition c);
 static GdkFilterReturn
 toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data);
@@ -188,6 +188,14 @@ convert_keyval(int key)
   case GDK_F11: return UKey_F11;
   case GDK_F12: return UKey_F12;
   case GDK_Multi_key: return UKey_Multi_key;
+  case GDK_Shift_L: return UKey_Shift_key;
+  case GDK_Shift_R: return UKey_Shift_key;
+  case GDK_Control_L: return UKey_Control_key;
+  case GDK_Control_R: return UKey_Control_key;
+  case GDK_Alt_L: return UKey_Alt_key;
+  case GDK_Alt_R: return UKey_Alt_key;
+  case GDK_Meta_L: return UKey_Meta_key;
+  case GDK_Meta_R: return UKey_Meta_key;
   }
 
   if(key >= GDK_KP_0 && key <= GDK_KP_9) {
@@ -351,6 +359,15 @@ focus_in(GtkIMContext *ic)
 
   focused_context = uic;
 
+
+  if(im_uim_fd < 0) {
+    im_uim_fd     = uim_helper_init_client_fd(im_uim_helper_disconnect_cb);
+    if (im_uim_fd >= 0) {
+      read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
+			       helper_read_cb, 0);
+    }
+  }
+
   uim_prop_list_update(uic->uc);
 
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
@@ -650,7 +667,6 @@ update_prop_label_cb(void *ptr, char *str)
 {
   GString *tmp = g_string_new("");
   g_string_printf(tmp, "prop_label_update\ncharset=UTF-8\n%s", str);
-  /*  g_print("label %s\n",tmp->str);*/
 
   uim_helper_send_message(im_uim_fd, tmp->str);
   g_string_free(tmp, TRUE);
@@ -848,7 +864,9 @@ static void
 cand_end_cb(void *ptr)
 {
   IMUIMContext *uic = ptr;
-  gtk_widget_hide(uic->cwin.top_win);
+  if(uic->cwin.top_win) {
+    gtk_widget_hide(uic->cwin.top_win);
+  }
   uic->cwin.is_active = 0;
 }
 
@@ -875,12 +893,11 @@ im_module_create (const gchar *context_id)
   }
 
   if(im_uim_fd < 0) {
-    im_uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);
+    im_uim_fd     = uim_helper_init_client_fd(im_uim_helper_disconnect_cb);
     if (im_uim_fd >= 0) {
       read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
 			       helper_read_cb, 0);
     }
-
   }
 
   uim_init();
@@ -995,6 +1012,7 @@ helper_read_cb(gpointer p, int fd, GdkInputCondition c)
 
   result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
   strcat(result, tmp);
+  free(tmp);
 
   if(uim_helper_str_terminated(result)) {
     im_uim_parse_helper_str(result);
@@ -1004,8 +1022,9 @@ helper_read_cb(gpointer p, int fd, GdkInputCondition c)
 }
 
 static void
-helper_disconnect_cb(void)
+im_uim_helper_disconnect_cb(void)
 {
+  im_uim_fd = -1;
   gdk_input_remove(read_tag);
 }
 
