@@ -12,6 +12,8 @@
 (define preedit-reverse 2)
 (define preedit-cursor 4)
 
+;; config
+(define enable-im-switch #f)
 
 (define make-im
   (lambda (name lang encoding init-arg init-context release-context
@@ -188,7 +190,17 @@
 	((c (find-context id))
 	 (im (context-im c))
 	 (handler (im-key-press-handler im)))
-      (handler id key state))))
+      (and
+       (if (and
+	    enable-im-switch
+	    (switch-im-key key state))
+	   (begin
+	     (switch-im id (im-name im)
+			#f)
+	     #t)
+	   (if (not (modifier-key key state))
+	       (handler id key state))
+	   #t)))))
 ;
 (define key-release-handler
   (lambda (id key state)
@@ -196,7 +208,8 @@
 	((c (find-context id))
 	 (im (context-im c))
 	 (handler (im-key-release-handler im)))
-      (handler id key state))))
+      (if (not (modifier-key key state))
+	  (handler id key state)))))
 
 (define reset-handler
   (lambda (id)
@@ -222,7 +235,26 @@
 	 (handler (im-prop-handler im)))
       (handler id prop))))
 
+(define find-im-by-name-rec
+  (lambda (name lst)
+    (if lst
+	(if (eq? (caar lst) name)
+	    lst
+	    (find-im-by-name-rec name (cdr lst)))
+	nil)))
 
+(define next-im
+  (lambda (name)
+    (let ((en (cdr (find-im-by-name-rec name im-list))))
+      (if en (caar en)
+	  (caar im-list)))))
+
+(define switch-im
+  (lambda (id name)
+    (release-context id)
+    (create-context id nil (next-im name))))
+
+;; find im by name and lang
 (define select-im
   (lambda (name lang)
     (let ((res ()))
