@@ -8,9 +8,7 @@
 #include "context.h"
 
 static FILE *primer = NULL, *primew = NULL;
-static int prime_running;
 static int prime_pid = 0;
-static int prime_active;
 
 static char buf[8192];
 
@@ -19,55 +17,21 @@ extern LISP sym_t;
 static char *prime_command = "prime";
 
 
-/**/
-pid_t open_pipe_rw(FILE ** fr, FILE ** fw);
-void  myExec(char *command);
-void  close_tty(void);
-
-void
-init_prime()
-{
-    prime_active = prime_running;
-    if (primer != NULL)
-	fclose(primer);
-    if (primew != NULL)
-	fclose(primew);
-    primer = primew = NULL;
-    if (prime_pid)
-	kill(prime_pid, SIGKILL);
-    prime_pid = 0;
-}
-
-static int
-open_prime(char *prime_command)
-{
-    prime_pid = open_pipe_rw(&primer, &primew);
-    if (prime_pid < 0)
-	goto err0;
-    if (prime_pid == 0) {
-	/* child */
-	myExec(prime_command);
-    }
-    return 1;
-  err0:
-    prime_pid = 0;
-    prime_active = prime_running = 0;
-    return 0;
-}
-
 static char *
 prime_send_command_internal(char *str)
 {
   char *tmp = strdup("");
  
   if (primer == NULL || primew == NULL)
-    if (open_prime(prime_command) == 0) {
-      free(tmp);
-      return NULL;
-    }
-
+    prime_pid = uim_ipc_open_command(prime_pid, &primer, &primew, prime_command );
+  
+  if(prime_pid == 0) {
+    free(tmp);
+    return NULL;
+  }
+  
   fprintf( primew, str );
-
+  
  again:
   if (fflush(primew) != 0) {
     switch (errno) {
@@ -94,8 +58,7 @@ prime_send_command_internal(char *str)
  err:
   /* XXX: backend prime is not working? */
   free(tmp);
-  init_prime();
-  prime_active = prime_running = 0;
+  prime_pid = uim_ipc_open_command(prime_pid, primer, primew, prime_command );
   return NULL;
   
 }
@@ -109,6 +72,8 @@ prime_send_command(LISP str_)
 
   result = prime_send_command_internal( str );
 
+  //  free(str);
+
   if(result == NULL)
     {
       return NIL;
@@ -118,11 +83,12 @@ prime_send_command(LISP str_)
 }
 
 static LISP
-init_prime_lib()
+prime_lib_init()
 {
-  int i;
-  init_prime();
-  open_prime("prime");
+  prime_pid = uim_ipc_open_command(prime_pid, primer, primew, prime_command );
+  if(prime_pid == 0) {
+    return NIL;
+  }
   return sym_t;
 }
 
@@ -213,79 +179,16 @@ prime_split_string(LISP _splittee, LISP _splitter)
    for(i = j; i >= 0 ; i--)
      {
        l = cons ( strcons( strlen(strs[i]), strs[i] ), l );
+       //     free(strs[i]);
      }
-   free(strs);
+   //   free(strs);
    return l;
 }
 
-pid_t
-open_pipe_rw(FILE ** fr, FILE ** fw)
-{
-    int fdr[2];
-    int fdw[2];
-    pid_t pid;
-
-    if (fr && pipe(fdr) < 0)
-	goto err0;
-    if (fw && pipe(fdw) < 0)
-	goto err1;
-
-    pid = fork();
-    if (pid < 0)
-	goto err2;
-    if (pid == 0) {
-	/* child */
-	if (fr) {
-	    close(fdr[0]);
-	    dup2(fdr[1], 1);
-	}
-	if (fw) {
-	    close(fdw[1]);
-	    dup2(fdw[0], 0);
-	}
-    }
-    else {
-	if (fr) {
-	    close(fdr[1]);
-	    if (*fr == stdin)
-		dup2(fdr[0], 0);
-	    else
-		*fr = fdopen(fdr[0], "r");
-	}
-	if (fw) {
-	    close(fdw[0]);
-	    if (*fw == stdout)
-		dup2(fdw[1], 1);
-	    else
-		*fw = fdopen(fdw[1], "w");
-	}
-    }
-    return pid;
-  err2:
-    if (fw) {
-	close(fdw[0]);
-	close(fdw[1]);
-    }
-  err1:
-    if (fr) {
-	close(fdr[0]);
-	close(fdr[1]);
-    }
-  err0:
-    return (pid_t) - 1;
-}
-
-void
-myExec(char *command)
-{
-    execl("/bin/sh", "sh", "-c", command, NULL);
-    exit(127);
-}
-
 void
 uim_init_prime()
 {
-  init_subr_0("prime-lib-init", init_prime_lib);
+  init_subr_0("prime-lib-init", prime_lib_init);
   init_subr_1("prime-lib-send-command", prime_send_command);
   init_subr_2("prime-string-split", prime_split_string);
 }
