@@ -53,7 +53,7 @@ static int init_serv_fd(char *path)
     perror("failed in listen()");
     return -1;
   }
-  fprintf(stderr,"listen at %s\n",path);
+  /*  fprintf(stderr,"listen at %s\n",path);*/
   return fd;
 }
 
@@ -90,14 +90,9 @@ free_client(struct client *cl)
   cl->fd = -1;
 }
 
-int uim_helper_server_new_context_id(void)
-{
-  context_id++;
-  return context_id;
-}
 
-
-static parse_content(char *content, struct client *cl)
+static void
+parse_content(char *content, struct client *cl)
 {
   char buf[BUFFER_SIZE];
   int i;
@@ -192,7 +187,7 @@ uim_helper_server_process_connection(int serv_fd,
       cl->fd = new_fd;
       /*write(cl->fd, "OK\n", 3);*/
 
-      fprintf(stderr,"accept new fd:%d\n",new_fd);
+      /*      fprintf(stderr,"accept new fd:%d\n",new_fd);*/
     }    
     
     /* check data from clients reached */
@@ -218,13 +213,17 @@ main(int argc, char **argv)
   char *path = uim_helper_get_pathname();
   int serv_fd;
   unlink(path);
-  serv_fd = init_serv_fd(path);
   init_client_array();
 
+  serv_fd = init_serv_fd(path);
+
+  printf("waiting\n\n");
+  fflush(stdout);
+
   if (serv_fd < 0) {
     return 0;
   }
-  fprintf(stderr,"Waiting for connection at %s\n", path);
+  /*  fprintf(stderr,"Waiting for connection at %s\n", path);*/
 
   free(path);
 
