@@ -47,6 +47,11 @@ extern "C" {
     UKey_F11,
     UKey_F12,
     UKey_Multi_key,
+    /* this part is especially unstable */
+    UKey_Shift_key,
+    UKey_Control_key,
+    UKey_Alt_key,
+    UKey_Meta_key,
     UKey_Other = 1000
   };
   
